unit uCTDSupport;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, forms, fileUtil, uBigIni, uTools, zStream, uStrTools;

function CTDCompress(const OriginalFname, Fname : String ) : String;
function CTDDecompress(const Fname : String) : String;
procedure CTDGetSectionList(const fname : String; var lst : TStringList);
procedure CTDSetSectionList(const fname : String; var lst : TStringList);
procedure CTDGetSection(const fname, SectionName : String; var lst : TStringList);
procedure CTDAddSection(const fname, SectionName : String; var SectionContent : TStringList);
procedure CTDDeleteSection(const fname, SectionName : String);
function CTDReadProperty(const fname, propName : String) : String;
procedure CTDSetProperty(const fname, PropName, PropValue : String);
Function CTDStripTags(const s : String) : String;
procedure CTDGetBookmarks(const fname : String; var lst : TStringList);
procedure CTDSetBookmarks(const fname : String; var lst : TStringList);


implementation


function CTDCompress(const OriginalFname, Fname : String ) : String;
var
   strm: TFileStream;
   comp: Tcompressionstream;
   outs: TFileStream;
begin
    if fileExists(fname) then
    begin
        if lowercase(extractFileExt(fname)) = '.stxt' then
        begin
           result := Fname;
           exit;
        end else if (lowercase(extractFileExt(fname)) = '.ctd') or (lowercase(extractFileExt(fname)) = '.~ctd') then
        begin
            // do compression
            strm := TFileStream.create(fname, fmOpenRead);
            try
               strm.position := 0;
               try
                     if fileExists(OriginalFname) then deleteFile(OriginalFname);
                     try
                     outs := TFileStream.Create(OriginalFname, fmCreate or fmSHAREdenyNone);
                         try
                              comp := Tcompressionstream.Create(clmax, outs, False);
                              try
                                   comp.CopyFrom(strm, strm.Size);
                                   comp.flush;
                              finally
                                     comp.Free;
                              end;
                         finally
                                outs.Free;
                         end;
                     except
                           //
                     end;
               finally
                      strm.free;
               end;
            except
                  // Error
            end;
            if fileExists(fname) then deleteFile(Fname);
            result := OriginalFname;
            exit;
        end;
    end else
    begin
        //
        result := fname;
        exit;
    end;
end;

function CTDDecompress(const Fname : String) : String;
var
   tfname : String;
   strm: TFileStream;
   comp: TDecompressionstream;
   lst : TStringList;
begin
    if fileExists(fname) then
    begin
        if lowercase(extractFileExt(fname)) = '.stxt' then
        begin
           tFname := BackSlash(MyGetTempPath) + ExtractFileName(ChangeFileExt(Fname, '.~ctd'));
           CopyFile(fname, Tfname, false);
           result := TFname;
           exit;
        end else if lowercase(extractFileExt(fname)) = '.ctd' then
        begin
           // do compression
           tFname := BackSlash(MyGetTempPath) + ExtractFileName(ChangeFileExt(Fname, '.~ctd'));
           try
               strm := TFileStream.Create(fname, fmOpenRead or fmSHAREDENYNONE);
               strm.position := 0;
               try
                   comp := Tdecompressionstream.Create(Strm);
                   try
                          if FileExists(TFname) then deleteFile(TFname);
                          comp.Position := 0;
                          lst := TStringList.create;
                          try
                             lst.LoadFromStream(comp);
                             lst.SaveToFile(TFname);
                          finally
                                 lst.free;
                          end;
                   finally
                     comp.Free;
                   end;
               finally
                      strm.Free;
               end;
           except
             // ignore
             // Raise something here
           end;
           result := TFname;
        end;
    end else
    begin
        result := fname;
    end;
end;

procedure CTDGetSectionList(const fname : String; var lst : TStringList);
var
   tfname : String;
   ini : TBigIniFile;
begin
    if not assigned(lst) then
    begin
         lst := TStringList.create;
    end;
    TFname := CTDDecompress(fname);
    ini := TBigIniFile.create(TFname);
    try
        ini.ReadnumberedList('Sections', lst, '');
    finally
           ini.free;
    end;
    if fileExists(TFname) then deleteFile(TFname);
end;

procedure CTDSetSectionList(const fname : String; var lst : TStringList);
var
   tfname : String;
   ini : TBigIniFile;
begin
    if not assigned(lst) then
    begin
         // Should raise, for now exit
       exit;
    end;
    TFname := CTDDecompress(fname);
    ini := TBigIniFile.create(TFname);
    try
        ini.WriteNumberedList('Sections', lst);
        ini.FlushFile;
    finally
           ini.free;
    end;
    CTDCompress(Fname, TFname);
    if fileExists(TFname) then deleteFile(TFname);
end;

procedure CTDGetSection(const fname, SectionName : String; var lst : TStringList);
var
   TFname : String;
   ini : TBigIniFile;
begin
    if not assigned(lst) then lst := TStringList.create;
    TFname := CTDDecompress(Fname);
    ini := TBigIniFile.create(TFname);
    try
        ini.ReadNumberedList('Section_' + SectionName, lst, '');
    finally
           ini.free;
    end;
    if FileExists(TFname) then DeleteFile(TFname);
end;

procedure CTDSetSection(const fname, SectionName : String; var lst : TStringList);
var
   TFname : String;
   ini : TBigIniFile;
begin
    if not assigned(lst) then lst := TStringList.create;
    TFname := CTDDecompress(Fname);
    ini := TBigIniFile.create(TFname);
    try
        ini.WriteNumberedList('Section_' + SectionName, lst);
        ini.FlushFile;
    finally
           ini.free;
    end;
    CTDCompress(fname, TFname);
    if FileExists(TFname) then DeleteFile(TFname);
end;

procedure CTDAddSection(const fname, SectionName : String; var SectionContent : TStringList);
var
   TFname : String;
   INI : TBigIniFile;
   lst : TStringList;
begin
    if not assigned(SectionContent) then
    begin
       // Raise
       exit;
    end;
    TFname := CTDDecompress(Fname);
    ini := TBigIniFile.create(TFname);
    try
        lst := TStringList.create;
        try
            ini.ReadNumberedList('Sections', lst, '');
            if lst.IndexOf(SectionName) = - 1 then
            begin
                 lst.add(SectionName);
                 ini.WriteNumberedList('Sections', lst);
            end;
            ini.WriteNumberedList('Section_' + SectionName, SectionContent);
            ini.FlushFile;
        finally
               lst.free;
        end;
    finally
           ini.free;
    end;
    CTDCompress(Fname, TFname);
    if FileExists(TFname) then deleteFile(TFname);
end;

procedure CTDDeleteSection(const fname, SectionName : String);
var
   TFname : String;
   ini : TBigIniFile;
   lst : TStringList;
begin
    TFname := CTDDecompresS(fname);
    ini := TBigIniFile.create(TFname);
    try
        lst := TStringList.create;
        try
            ini.ReadNumberedList('Sections', lst, '');
            if lst.IndexOf(SectionName)<>-1 then
            begin
                 lst.delete(lst.IndexOf(SectionName));
                 ini.EraseSection('Section_' + SectionName);
                 ini.WriteNumberedList('Sections', lst);
                 ini.FlushFile;
            end;
        finally
               lst.free;
        end;
    finally
           ini.free;
    end;
    CTDCompress(Fname, TFname);
    if fileExists(TFname) then DeleteFile(TFname);
end;

{ Valid Properties
        Title,
        Author,
        Website,
        Email,
        Subject,
        Category,
        Tags,
        Rating,
        Genre,
        Summary,
        GUIID,
        Notes - converted to a string and returned
}

function CTDReadProperty(const fname, propName : String) : String;
var
   TFname : String;
   ini : TBigIniFile;
   lst : TStringList;
begin
    TFname := CTDDecompress(Fname);
    ini := TBigIniFile.create(TFname);
    try
        if UPPERCASE(PropName) = 'PROPERTIES_NOTES' then
        begin
             lst :=  TStringList.create;
             try
                ini.ReadNumberedList('Properties_Notes', lst, '');
                result := lst.text;
             finally
                    lst.free;
             end;
        end else
        begin
             result := ini.ReadString('Properties', PropName, '');
        end;

    finally
           ini.free;
    end;
    if fileExists(TFname) then deleteFile(TFname);
end;

procedure CTDSetProperty(const fname, PropName, PropValue : String);
var
   TFname : String;
   ini : TBigIniFile;
   lst : TStringList;
begin
     TFname := CTDDecompress(Fname);
     ini := TBigIniFile.create(TFname);
     try
         if UPPERCASE(PropName) = 'PROPERTIES_NOTES' then
         begin
              lst :=  TStringList.create;
              try
                 lst.text := PropValue;
                 ini.WriteNumberedList('Properties_Notes', lst);
              finally
                     lst.free;
              end;
         end else
         begin
             ini.WriteString('Propeties', PropName, PropValue);
         end;
         ini.flushFile;
     finally
            ini.free;
     end;
     CTDCompress(Fname, TFname);
     If FileExists(TFname) then DeleteFile(TFname);
end;

Function CTDStripTags(const s : String) : String;
var
   txt : String;
begin
  txt := s;
  txt := stringReplaceAll(txt, '[title]', '');
  txt := stringReplaceAll(txt, '[/title]', '');
  txt := stringReplaceAll(txt, '[H1]', '');
  txt := stringReplaceAll(txt, '[/H1]', '');
  txt := stringReplaceAll(txt, '[H2]', '');
  txt := stringReplaceAll(txt, '[/H2]', '');
  txt := stringReplaceAll(txt, '[H3]', '');
  txt := stringReplaceAll(txt, '[/H3]', '');
  txt := stringReplaceAll(txt, '[BR]', '');
  txt := stringReplaceAll(txt, '[H4]', '');
  txt := stringReplaceAll(txt, '[/H4]', '');
  txt := stringReplaceAll(txt, '[QUOTE]', '');
  txt := stringReplaceAll(txt, '[/QUOTE]', '');
  txt := stringReplaceAll(txt, '[MONO]', '');
  txt := stringReplaceAll(txt, '[/MONO]', '');
  txt := stringReplaceAll(txt, '[THOUGHT]', '');
  txt := stringReplaceAll(txt, '[/THOUGHT]', '');
  txt := stringReplaceAll(txt, '[SPEECH]', '');
  txt := stringReplaceAll(txt, '[/SPEECH]', '');
  txt := stringReplaceAll(txt, '[B]', '');
  txt := stringReplaceAll(txt, '[/B]', '');
  txt := stringReplaceAll(txt, '[I]', '');
  txt := stringReplaceAll(txt, '[/I]', '');
  txt := stringReplaceAll(txt, '[U]', '');
  txt := stringReplaceAll(txt, '[/U]', '');
  txt := stringReplaceAll(txt, '[LEFT]', '');
  txt := stringReplaceAll(txt, '[/LEFT]', '');
  txt := stringReplaceAll(txt, '[RIGHT]', '');
  txt := stringReplaceAll(txt, '[/RIGHT]', '');
  txt := stringReplaceAll(txt, '[CENTER]', '');
  txt := stringReplaceAll(txt, '[/CENTER]', '');
  txt := stringReplaceAll(txt, '[SCENE]', '');
  txt := stringReplaceAll(txt, '[/SCENE]', '');
  txt := stringReplaceAll(txt, '[BREAK]', '');
  result := txt;
end;

procedure CTDGetBookmarks(const fname : String; var lst : TStringList);
var
   TFname : String;
   ini : TBigIniFile;
begin
     if not assigned(lst) then lst := TStringList.create;
     TFname := CTDDecompress(Fname);
     ini := TBigIniFile.create(TFname);
     try
         ini.ReadNumberedList('Favorites', lst, '');
     finally
            ini.free;
     end;
     if FileExists(TFname) then DeleteFile(TFname);
end;

procedure CTDSetBookmarks(const fname : String; var lst : TStringList);
var
   TFname : String;
   ini : TBigIniFile;
begin
     if not assigned(lst) then exit;
     TFname := CTDDecompress(Fname);
     ini := TBigIniFile.create(TFname);
     try
         ini.WriteNumberedList('Favorites', lst);
         ini.flushFile;
     finally
            ini.free;
     end;
     CTDCompress(Fname, TFname);
     if FileExists(TFname) then DeleteFile(TFname);
end;

end.

